package com.ibatis.common.xml;



public class NodeletException extends Exception {

  public NodeletException() {
    super();
  }

  public NodeletException(String msg) {
    super(msg);
  }

  public NodeletException(Throwable cause) {
    super(cause);
  }

  public NodeletException(String msg, Throwable cause) {
    super(msg, cause);
  }

}
